import Flutter
import BradySdk
import UIKit
import Foundation

public class BradySdkApi: PrinterDiscoveryListener, PrinterUpdateListener {
   
    lazy private var printerDiscovery: PrinterDiscovery = PrinterDiscoveryFactory.getPrinterDiscovery(listeners: listeners)
    private var printerDetails: PrinterDetails?
    private var printingOptions: PrintingOptions = PrintingOptions()
    private var listeners: [PrinterDiscoveryListener]
    private var foundPrinters = [DiscoveredPrinterInformation]()
    private var currentTemplate: Template?
    private var currentTemplateName: String = ""
    private var currentBitmap: UIImage?
    private var isConnected: Bool? = nil
    private var printingStatus: PrintingStatus? = nil
    private var printerUpdateAvailable: Bool = false
    private var newPrinterUpdates: [String] = []
    
    init(){
        listeners = [PrinterDiscoveryListener]()
        listeners.append(self)
    }
    
    public func startBlePrinterDiscovery() {
        printerDiscovery.startBlePrinterDiscovery()
    }

    public func startWifiPrinterDiscovery() {
        printerDiscovery.startWifiPrinterDiscovery()
    }
    
    public func stopPrinterDiscovery() {
        printerDiscovery.stopBlePrinterDiscovery()
    }
    
    public func getPrinters() -> String {
        var printerNames: String = ""
        for printer in foundPrinters {
            printerNames += printer.getName() + ", "
        }
        return printerNames
    }
    
    public func connectToBluetoothPrinter(printer: String?) async -> Bool {
        
        var printerObject: DiscoveredPrinterInformation? = nil
        for i in foundPrinters {
            if i.getName() == printer && i.getConnectionType() == ConnectionType.BLE {
                printerObject = i
            }
        }
        
        if printerObject == nil {
            return false
        }
        
        return await connect(printerObject: printerObject!)
    }

    public func connectToWifiPrinter(printer: String?) async -> Bool {

        var printerObject: DiscoveredPrinterInformation? = nil
        for i in foundPrinters {
            if i.getName() == printer && i.getConnectionType() == ConnectionType.Wifi {
                printerObject = i
            }
        }

        if printerObject == nil {
            return false
        }
        
        return await connect(printerObject: printerObject!)
    }
    
    private func connect(printerObject: DiscoveredPrinterInformation) async -> Bool {
        var listeners: Array<PrinterUpdateListener> = []
        listeners.append(self)
        forgetLastConnectedPrinter()
        printerDetails = await printerDiscovery.connectToDiscoveredPrinter(printerSelected: printerObject, listeners: listeners)
        if printerDetails != nil {
            return true
        }
        else {
            return false
        }
    }
    
    public func isPrinterBluetooth(printerName: String?) -> Bool?{
        for printer in foundPrinters {
            if printer.getName() == printerName && printer.getConnectionType() == ConnectionType.BLE {
                return true
            }
            else if printer.getName() == printerName && printer.getConnectionType() == ConnectionType.Wifi {
                return false
            }
        }
        return nil
    }

    public func getPrinterStatus() -> String {
        let enumValue: PrinterStatus = PrinterStatus(rawValue: (self.printerDetails?.getPrinterStatus().rawValue) ?? "Disconnected")!
        let printerStatus = enumValue.rawValue
        return printerStatus
    }

    public func getPrinterStatusMessage() -> String {
        return self.printerDetails?.getPrinterStatusMessage() ?? ""
    }

    public func getPrinterStatusMessageTitle() -> String {
        return self.printerDetails?.getPrinterStatusMessageTitle() ?? ""
    }

    public func getPrinterStatusRemedyExplanationMessage() -> String {
        return self.printerDetails?.getPrinterStatusRemedyExplanationMessage() ?? ""
    }

    public func getPrinterName() -> String {
        let printerName = self.printerDetails?.getPrinterName() ?? "null"
        return printerName
    }

    public func getPrinterModel() -> String {
        let printerModel = self.printerDetails?.getPrinterModel() ?? "null"
        return printerModel
    }

    public func getConnectionType() -> String {
        return self.printerDetails?.getConnectionType()?.rawValue ?? "null"
    }

    public func getBatteryLevelPercentage() -> Int {
        return Int(self.printerDetails?.getBatteryLevelPercentage() ?? 0)
    }

    public func getIsAcConnected() -> Bool {
        if self.printerDetails != nil {
            return self.printerDetails!.getIsAcConnected()
        }
        return false
    }

    public func getSupplyName() -> String {
        let supplyName = self.printerDetails?.getSupplyName() ?? "null"
        return supplyName
    }

    public func getSupplyRemainingPercentage() -> Int {
        return self.printerDetails?.getSupplyRemainingPercentage() ?? 0
    }

    public func getSupplyWidth() -> Double {
        return self.printerDetails?.getSupplyWidth() ?? 0
    }

    public func getSupplyHeight() -> Double {
        return self.printerDetails?.getSupplyHeight() ?? 0
    }

    public func getIsSupplyPreSized() -> Bool {
        return self.printerDetails?.getIsSupplyPreSized() ?? false
    }

    public func checkForPartMismatch() -> Bool {
        do {
            if self.currentTemplate != nil && self.printerDetails != nil {
                return try self.printerDetails?.checkForPartMismatch(template: self.currentTemplate!) ?? true
            }
            else {
                return true
            }
        }
        catch {
            return true
        }
    }
    
    public func getLastConnectedPrinterName() -> String? {
        return self.printerDiscovery.getLastConnectedPrinterName()
    }
    
    public func setTemplate(dataString: String, fileExtension: String?) {
        let base64Data = dataString.data(using: .utf8)
        if base64Data == nil {
            Swift.print("Could not get the desired Template object.")
            return
        }
        let templateData: Data? = Data(base64Encoded: base64Data!)
        if templateData == nil {
            Swift.print("Could not get the desired Template object.")
            return
        }
        let iStream: InputStream = InputStream(data: templateData!)
        if fileExtension != nil && fileExtension != "" && 
            fileExtension!.uppercased() != "BWT" && fileExtension!.uppercased() != "BWS" {
            guard let data = Data(base64Encoded: dataString) else { return }
            self.currentBitmap = UIImage(data: data)
        }
        else {
            self.currentTemplate = TemplateFactory.getTemplate(template: iStream)
        }
    }

    public func getHaveOwnership() -> Bool {
        if self.printerDetails != nil && self.printerDetails?.getPrinterModel() == "M211" {
            return self.printerDiscovery.getHaveOwnership() ?? false
        }
        else {
            return false
        }
    }

    public func getTemplateDataNames() -> String {
        var templateDataNames = ""
        if self.currentTemplate != nil {
            do {
                for obj in try self.currentTemplate!.getTemplateData() {
                    templateDataNames += obj.getName() + ", "
                }
            }
            catch {
                return ""
            }
        }
        else {
            Swift.print("Template is null (use setTemplate(inputStream:) first)!")
        }
        return templateDataNames
    }

    public func getTemplateSupplyName() -> String? {
        if self.currentTemplate != nil {
            return self.currentTemplate!.getPartInfo()
        }
        else {
            return nil
        }
    }

    public func setPlaceholderValue(placeholder: String, value: String) {
        if self.currentTemplate != nil {
            do {
                for obj in try self.currentTemplate!.getTemplateData() {
                    if obj.getName() == placeholder {
                        try obj.setValue(value: value)
                    }
                }
            }
            catch {
                return
            }
        }
        else {
            Swift.print("Template is null (use setTemplate(inputStream:) first)!")
        }
    }

    public func getPreview(elementSize: Int) -> String {
        do {
            if self.currentBitmap == nil {
                let image: UIImage = UIImage(cgImage: (try self.currentTemplate?.getPreview(labelNumber: 0, dpi: 96, maxPixelWidthAndHeight: Double(elementSize)))!)
                let imageString = image.pngData()?.base64EncodedString() ?? "null"
                return imageString
            }
            else {
                let imageString = currentBitmap!.pngData()?.base64EncodedString() ?? "null"
                return imageString
            }
        }
        catch {
            Swift.print("Could not retrieve the desired Template to preview.")
            return "null"
        }
    }

    private func setPrintingOptions(_ cutAfterEachLabel: Bool, _ numOfCopies: Int, _ isCollated: Bool) {
        if cutAfterEachLabel {
            self.printingOptions.cutOption = CutOption.EndOfLabel
        }
        else if !cutAfterEachLabel {
            self.printingOptions.cutOption = CutOption.EndOfJob
        }
        else {
            self.printingOptions.cutOption = CutOption.Never
        }
        self.printingOptions.numberOfCopies = Int(numOfCopies)
        self.printingOptions.isCollated = isCollated
    }
    
    public func print(copies: Int, cutAfterEachLabel: Bool, isCollated: Bool, printTrailer: Bool) async -> Bool {
        if self.currentTemplate != nil && self.printerDetails != nil {
            setPrintingOptions(cutAfterEachLabel, copies, isCollated)
            Swift.print("Printing Template...")
            self.printingStatus = await (printerDetails?.print(template: self.currentTemplate!, printingOptions: self.printingOptions, dontPrintTrailerFlag: !printTrailer)) ?? PrintingStatus.PrintingFailed
        }
        else if self.currentBitmap != nil && self.printerDetails != nil {
            self.printingStatus = await (printerDetails?.print(image: self.currentBitmap!, printingOptions: self.printingOptions, dontPrintTrailerFlag: false)) ?? PrintingStatus.PrintingFailed
        }

        if printingStatus == PrintingStatus.PrintingSucceeded {
            Swift.print("Print Succeeded!")
            return true
        }
        else {
            Swift.print("Print Failed!")
            return false
        }
    }

    public func printBitmapWithDimensions(bitmapWidthInInches: Double, labelLengthInInches: Double, copies: Int, cutAfterEachLabel: Bool, isCollated: Bool, printTrailer: Bool) async -> Bool {
        if self.currentTemplate != nil && self.printerDetails != nil {
            setPrintingOptions(cutAfterEachLabel, copies, isCollated)
            Swift.print("Printing Template...")
            self.printingStatus = await (printerDetails?.print(template: self.currentTemplate!, printingOptions: self.printingOptions, dontPrintTrailerFlag: !printTrailer)) ?? PrintingStatus.PrintingFailed
        }
        else if self.currentBitmap != nil && self.printerDetails != nil {
            self.currentBitmap = UIImage(named: self.currentTemplateName) ?? UIImage()
            self.printingStatus = await (printerDetails?.print(image: self.currentBitmap!, imageWidth: bitmapWidthInInches, labelLength: labelLengthInInches, printingOptions: self.printingOptions, dontPrintTrailerFlag: false)) ?? PrintingStatus.PrintingFailed
        }

        if printingStatus == PrintingStatus.PrintingSucceeded {
            Swift.print("Print Succeeded!")
            return true
        }
        else {
            Swift.print("Print Failed!")
            return false
        }
    }
    
    public func feed() async -> Bool {
        if printerDetails != nil {
            let fed = await printerDetails!.feed() ?? false
            return fed
        }
        Swift.print("Feed failed because there is no connection to the desired printer!")
        return false
    }
    
    public func cut() async -> Bool {
        if printerDetails != nil {
            let cut = await printerDetails!.cutLabel() ?? false
            return cut
        }
        Swift.print("Cut failed because there is no connection to the desired printer!")
        return false
    }

    public func getLastConnectedPrinter() -> String? {
        return UserDefaults.standard.string(forKey: "LAST_CONNECTED_PRINTER_NAME")
    }
    
    public func disconnect() async -> Bool {
        if printerDetails != nil {
            let result = await printerDetails!.disconnect() ?? false
            if result {
                Swift.print("Disconnect Successful!")
                self.printerDetails = nil
                return true
            }
            else {
                Swift.print("Disconnect Failed!")
                return false
            }
        }
        Swift.print("Disconnect failed because there is no connection to the desired printer!")
        return false
    }

    public func disconnectWithoutForget() async -> Bool {
        if printerDetails != nil {
            let result = await self.printerDetails?.disconnectWithoutForget() ?? false
            if result {
                Swift.print("Disconnect Successful!")
                self.printerDetails = nil
                return true
            }
            else {
                Swift.print("Disconnect Failed!")
                return false
            }
        }
        Swift.print("Disconnect failed because there is no connection to the desired printer!")
        return false
    }

    public func forgetLastConnectedPrinter() {
        let preferences = UserDefaults.standard
        preferences.set(nil, forKey: "LAST_CONNECTED_PRINTER_NAME")
        preferences.set(nil, forKey: "LAST_CONNECTED_PRINTER_OBJET")
    }
    
    public func printerDiscovered(discoveredPrinterInformation: DiscoveredPrinterInformation) {
        if(!foundPrinters.contains(where: {discoveredPrinterInformation.getName() == $0.getName()})) {
            foundPrinters.append(discoveredPrinterInformation)
            Swift.print(discoveredPrinterInformation.getName() + " printer discovered with protocol: " + discoveredPrinterInformation.getConnectionType().rawValue)
        }
    }
    
    public func printerDiscoveryStarted() {
        
    }
    
    public func printerDiscoveryStopped() {
        
    }
    
    public func PrinterUpdate(changedProperties: Array<BradySdk.PrinterProperties>) {
        for property in changedProperties {
            Swift.print(String(describing: property) + " Property Updated!")
            if !newPrinterUpdates.contains(where: {$0 == String(describing: property)}) {
                newPrinterUpdates.append(String(describing: property))
            }
        }
        printerUpdateAvailable = true
    }

     public func getAvailablePrinterUpdates() -> String {
        if printerUpdateAvailable {
            let tempUpdateList = self.newPrinterUpdates
            self.newPrinterUpdates = []
            printerUpdateAvailable = false

            var updateString = ""
            for update in tempUpdateList {
                updateString += update + ", "
            }
            return updateString
        }
        else {
            return ""
        }
    }
}
