import 'dart:async';
import 'brady_flutter_plugin_platform_interface.dart';

class BradyFlutterPlugin {
  /// startPrinterDiscovery() starts the Bluetooth scan for nearby Brady Printers. All of the details about these devices are stored internally and can be retrieved using the getPrinters() API method.
  void startBlePrinterDiscovery() {
    BradyFlutterPluginPlatform.instance.startBlePrinterDiscovery();
  }

  /// startWifiPrinterDiscovery() starts the Wifi scan for nearby Brady Printers. All of the details about these devices are stored internally and can be retrieved using the getPrinters() API method.
  void startWifiPrinterDiscovery() {
    BradyFlutterPluginPlatform.instance.startWifiPrinterDiscovery();
  }

  /// stopPrinterDiscovery() stops all discovery scans that were previously started.
  void stopPrinterDiscovery() {
    BradyFlutterPluginPlatform.instance.stopPrinterDiscovery();
  }

  /// getPrinters() returns a List of Strings representing the printer names that have been discovered via the discovery scan.
  Future<List<String?>> getPrinters() async {
    return await BradyFlutterPluginPlatform.instance.getPrinters();
  }

  /// connectToBluetoothPrinter(String) attempts to connect to a printer via Bluetooth Low Energy whose name is passed into the method. This is asynchronous and returns a Boolean value afterwards with the result of the connection (i.e. success or failure).
  Future<bool?> connectToBluetoothPrinter(String? printer) async {
    return await BradyFlutterPluginPlatform.instance
        .connectToBluetoothPrinter(printer);
  }

  /// connectToWifiPrinter(String) attempts to connect to a printer via Wi-Fi whose name is passed into the method. This is asynchronous and returns a Boolean value afterwards with the result of the connection (i.e. success or failure).
  Future<bool?> connectToWifiPrinter(String? printer) async {
    return await BradyFlutterPluginPlatform.instance
        .connectToWifiPrinter(printer);
  }

  /// getPrinterStatus() returns a String representing the current printer connection status.
  Future<String?> getPrinterStatus() async {
    return await BradyFlutterPluginPlatform.instance.getPrinterStatus();
  }

  /// isPrinterBluetooth(String) takes the name of a discovered printer and checks if it was discovered by the Bluetooth Scan
  ///
  /// If true, it was discovered by the Bluetooth Scan and supports BLE connectivity.
  /// If false, it was discovered by Wi-Fi and support Wi-Fi connectivity.
  /// If null, the printer name passed in is not valid.
  Future<bool?> isPrinterBluetooth(String printerName) async {
    return await BradyFlutterPluginPlatform.instance
        .isPrinterBluetooth(printerName);
  }

  /// getPrinterStatusMessage() returns a String representing the current printer connection status message.
  Future<String?> getPrinterStatusMessage() async {
    return await BradyFlutterPluginPlatform.instance.getPrinterStatusMessage();
  }

  /// getPrinterStatusMessageTitle() returns a String representing the current printer connection status message title.
  Future<String?> getPrinterStatusMessageTitle() async {
    return await BradyFlutterPluginPlatform.instance
        .getPrinterStatusMessageTitle();
  }

  /// getPrinterStatusRemedyExplanationMessage() returns a String representing a solution to resolve the current printer error if there is one.
  Future<String?> getPrinterStatusRemedyExplanationMessage() async {
    return await BradyFlutterPluginPlatform.instance
        .getPrinterStatusRemedyExplanationMessage();
  }

  /// getPrinterName() returns a String representing the current connected printer's name.
  Future<String?> getPrinterName() async {
    return await BradyFlutterPluginPlatform.instance.getPrinterName();
  }

  /// getPrinterModel() returns a String representing the current connected printer's model.
  Future<String?> getPrinterModel() async {
    return await BradyFlutterPluginPlatform.instance.getPrinterModel();
  }

  /// getConnectionType() returns a String representing the protocol that was used to connect to the printer.
  Future<String?> getConnectionType() async {
    return await BradyFlutterPluginPlatform.instance.getConnectionType();
  }

  /// getBatteryLevelPercentage() returns a String representing the battery level percentage of the connected printer.
  Future<int?> getBatteryLevelPercentage() async {
    return await BradyFlutterPluginPlatform.instance
        .getBatteryLevelPercentage();
  }

  /// getIsAcConnected() returns a Boolean representing if the connect printer is charging.
  Future<bool?> getIsAcConnected() async {
    return await BradyFlutterPluginPlatform.instance.getIsAcConnected();
  }

  /// getSupplyName() returns a String representing the supply installed in the connected printer.
  Future<String?> getSupplyName() async {
    return await BradyFlutterPluginPlatform.instance.getSupplyName();
  }

  /// getSupplyRemainingPercentage() returns a String representing the percentage of supply remaining in the connected printer
  Future<int?> getSupplyRemainingPercentage() async {
    return await BradyFlutterPluginPlatform.instance
        .getSupplyRemainingPercentage();
  }

  /// getSupplyWidth() returns a String representing the width of the supply installed in the connected printer.
  Future<double?> getSupplyWidth() async {
    return await BradyFlutterPluginPlatform.instance.getSupplyWidth();
  }

  /// getSupplyHeight() returns a String representing the height of the supply installed in the connected printer.
  Future<double?> getSupplyHeight() async {
    return await BradyFlutterPluginPlatform.instance.getSupplyHeight();
  }

  /// getIsSupplyPreSized() returns a Boolean representing if the supply installed in the connected printer is pre-sized. If false, the supply is continuous.
  Future<bool?> getIsSupplyPreSized() async {
    return await BradyFlutterPluginPlatform.instance.getIsSupplyPreSized();
  }

  /// checkForPartMismatch() returns a Boolean representing if the selected template was not designed with the supply that is installed in the connected printer.
  Future<bool?> checkForPartMismatch() async {
    return await BradyFlutterPluginPlatform.instance.checkForPartMismatch();
  }

  /// getLastConnectedPrinterName() returns a String representing the name of the last printer that the app connected to.
  ///
  /// This is available to allow automatic connection. Since most users own and use only one printer, this can be called immediately after startPrinterDiscovery().
  /// Therefore, when a printer is discovered with the same name as the "last connected printer" you may choose to call the connectToPrinter(String) method to connect to the familiar printer immediately at start up.
  Future<String?> getLastConnectedPrinterName() async {
    return await BradyFlutterPluginPlatform.instance
        .getLastConnectedPrinterName();
  }

  /// setTemplate(String) takes a file path (i.e. 'assets/MyTemplate.BWT' or 'assets/MyImage.PNG') as a string representing a template file or image.
  ///
  /// This method sets the current template or image being acted upon when setPlaceholderValue(String, String) or print() is called.
  /// Therefore, calling setTemplate(String) again will override any previous template that has been set.
  Future<void> setTemplate(String filePath) async {
    return BradyFlutterPluginPlatform.instance.setTemplate(filePath);
  }

  ///getHaveOwnership() returns a Boolean representing the mobile device's ownership of a nearby M211.
  ///
  /// This will only ever return True when communicating with an M211. It is possible
  /// to fail a connection to an M211, but successfully gain ownership, hence this method's existence.
  Future<bool?> getHaveOwnership() async {
    return await BradyFlutterPluginPlatform.instance.getHaveOwnership();
  }

  /// getTemplateDataNames returns all object names on the current template.
  ///
  /// This can be used to see placeholder names instead of opening Brady Workstation
  /// to see what they are named. These object names can be directly passed into the
  /// setPlaceholderValue to set the value of a placeholder.
  Future<List<String?>> getTemplateDataNames() async {
    return BradyFlutterPluginPlatform.instance.getTemplateDataNames();
  }

  /// getTemplateSupplyName() returns the name of the supply that used to design the selected template
  Future<String?> getTemplateSupplyName() async {
    return await BradyFlutterPluginPlatform.instance.getTemplateSupplyName();
  }

  /// setPlaceholderValue(String, String) takes two Strings to set a placeholder with a specified value.
  ///
  /// The first argument must be a String which matches the placeholder's name in Brady Workstation where the template was created.
  /// The second argument must be a String representing the value you want to set to the placeholder.
  void setPlaceholderValue(String placeholder, String value) {
    return BradyFlutterPluginPlatform.instance
        .setPlaceholderValue(placeholder, value);
  }

  ///getPreview(Int) takes an Integer that represents the width in screen pixels that the print preview should display as (on the UI).
  Future<String?> getPreview(int elementSize) async {
    return await BradyFlutterPluginPlatform.instance.getPreview(elementSize);
  }

  /// print() asynchronously sends a print job of the currently set template to the printer that the app is currently connected to and returns a Boolean value afterwards with the result of the print operation (i.e. success or failure).
  Future<bool?> print(int copies, bool cutAfterEachLabel, bool isCollated,
      bool printTrailer) async {
    return await BradyFlutterPluginPlatform.instance
        .print(copies, cutAfterEachLabel, isCollated, printTrailer);
  }

  /// printBitmapWithDimensions() asynchronously sends a print job of the currently set template or bitmap image to the printer that the app is currently connected to and returns a Boolean value afterwards with the result of the print operation (i.e. success or failure).
  Future<bool?> printBitmapWithDimensions(
      double bitmapWidthInInches,
      double labelLengthInInches,
      int copies,
      bool cutAfterEachLabel,
      bool isCollated,
      bool printTrailer) async {
    return await BradyFlutterPluginPlatform.instance.printBitmapWithDimensions(
        bitmapWidthInInches,
        labelLengthInInches,
        copies,
        cutAfterEachLabel,
        isCollated,
        printTrailer);
  }

  /// feed() asynchronously makes the connected printer print one empty label and returns a Boolean value afterwards with the result of the feed operation(i.e. success or failure).
  Future<bool?> feed() async {
    return await BradyFlutterPluginPlatform.instance.feed();
  }

  /// cut() asynchronously makes the connected printer perform a cut operation (this can be called right after feed() for testing purposes) and returns a Boolean value afterwards with the result of the cut operation (i.e. success or failure).
  Future<bool?> cut() async {
    return await BradyFlutterPluginPlatform.instance.cut();
  }

  /// disconnect() asynchronously disconnects from the currently connected printer and returns a Boolean value afterwards with the result of the disconnect operation (i.e. success or failure).
  ///
  /// This will also clear the value of "LastConnectedPrinterName" internally.
  Future<bool?> disconnect() async {
    return await BradyFlutterPluginPlatform.instance.disconnect();
  }

  /// disconnectWithoutForget() asynchronously disconnects from the currently connected printer and returns a Boolean value afterwards with the result of the disconnect operation (i.e. success or failure).
  ///
  /// This will NOT clear the value of "LastConnectedPrinterName" internally.
  Future<bool?> disconnectWithoutForget() async {
    return await BradyFlutterPluginPlatform.instance.disconnectWithoutForget();
  }

  /// forgetLastConnectedPrinter() clears the value of "LastConnectedPrinterName internally preventing auto-connections.
  void forgetLastConnectedPrinter() {
    return BradyFlutterPluginPlatform.instance.forgetLastConnectedPrinter();
  }

  /// getAvailablePrinterUpdates() returns a list of string representing all printer updates that occurred since the last time this method was called.
  Future<List<String?>> getAvailablePrinterUpdates() async {
    return await BradyFlutterPluginPlatform.instance
        .getAvailablePrinterUpdates();
  }
}
