import 'dart:convert';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'brady_flutter_plugin_platform_interface.dart';

/// An implementation of [BradyFlutterPluginPlatform] that uses method channels.
class MethodChannelBradyFlutterPlugin extends BradyFlutterPluginPlatform {
  /// The method channel used to interact with the native platform.
  @visibleForTesting
  final methodChannel = const MethodChannel('brady_flutter_plugin');

  /// startBlePrinterDiscovery() starts the Bluetooth scan for nearby Brady Printers. All of the details about these devices are stored internally and can be retrieved using the getPrinters() API method.
  @override
  void startBlePrinterDiscovery() {
    methodChannel.invokeMethod<String>('startBlePrinterDiscovery');
  }

  /// startWifiPrinterDiscovery() starts the Bluetooth scan for nearby Brady Printers. All of the details about these devices are stored internally and can be retrieved using the getPrinters() API method.
  @override
  void startWifiPrinterDiscovery() {
    methodChannel.invokeMethod<String>('startWifiPrinterDiscovery');
  }

  /// stopPrinterDiscovery() stops all discovery scans that were previously started.
  @override
  void stopPrinterDiscovery() {
    methodChannel.invokeMethod<String>('stopPrinterDiscovery');
  }

  /// getPrinters() returns a List of Strings representing the printer names that have been discovered via the discovery scan.
  @override
  Future<List<String?>> getPrinters() async {
    String? printerNames =
        await methodChannel.invokeMethod<String>('getPrinters');
    List<String> printerNamesList = printerNames!.split(', ');
    printerNamesList.removeLast();
    return printerNamesList;
  }

  /// isPrinterBluetooth(String) takes the name of a discovered printer and checks if it was discovered by the Bluetooth Scan
  ///
  /// If true, it was discovered by the Bluetooth Scan and supports BLE connectivity.
  /// If false, it was discovered by Wi-Fi and support Wi-Fi connectivity.
  /// If null, the printer name passed in is not valid.
  @override
  Future<bool?> isPrinterBluetooth(String printerName) async {
    var sendMap = <String, dynamic>{"printerName": printerName};
    bool? isPrinterBluetooth =
        await methodChannel.invokeMethod<bool?>('isPrinterBluetooth', sendMap);
    return isPrinterBluetooth;
  }

  /// connectToBluetoothPrinter(String) attempts to connect to a printer via Bluetooth Low Energy whose name is passed into the method. This is asynchronous and returns a Boolean value afterwards with the result of the connection (i.e. success or failure).
  @override
  Future<bool?> connectToBluetoothPrinter(String? printer) async {
    var sendMap = <String, dynamic>{"printerName": printer};
    bool? connected = await methodChannel.invokeMethod<bool>(
        'connectToBluetoothPrinter', sendMap);
    return connected;
  }

  /// connectToWifiPrinter(String) attempts to connect to a printer via Wi-Fi whose name is passed into the method. This is asynchronous and returns a Boolean value afterwards with the result of the connection (i.e. success or failure).
  @override
  Future<bool?> connectToWifiPrinter(String? printer) async {
    var sendMap = <String, dynamic>{"printerName": printer};
    bool? connected =
        await methodChannel.invokeMethod<bool>('connectToWifiPrinter', sendMap);
    return connected;
  }

  /// getPrinterStatus() returns a String representing the current printer connection status.
  @override
  Future<String?> getPrinterStatus() async {
    String? printerStatus =
        await methodChannel.invokeMethod<String>('getPrinterStatus');
    return printerStatus;
  }

  /// getPrinterStatusMessage() returns a String representing the current printer connection status message.
  @override
  Future<String?> getPrinterStatusMessage() async {
    String? printerStatusMessage =
        await methodChannel.invokeMethod<String>('getPrinterStatusMessage');
    return printerStatusMessage;
  }

  /// getPrinterStatusMessageTitle() returns a String representing the current printer connection status message title.
  @override
  Future<String?> getPrinterStatusMessageTitle() async {
    String? printerStatusMessageTitle = await methodChannel
        .invokeMethod<String>('getPrinterStatusMessageTitle');
    return printerStatusMessageTitle;
  }

  /// getPrinterStatusRemedyExplanationMessage() returns a String representing a solution to resolve the current printer error if there is one.
  @override
  Future<String?> getPrinterStatusRemedyExplanationMessage() async {
    String? printerStatusRemedyExplanationMessage = await methodChannel
        .invokeMethod<String>('getPrinterStatusRemedyExplanationMessage');
    return printerStatusRemedyExplanationMessage;
  }

  /// getPrinterName() returns a String representing the current connected printer's name.
  @override
  Future<String?> getPrinterName() async {
    String? printerName =
        await methodChannel.invokeMethod<String>('getPrinterName');
    return printerName;
  }

  /// getPrinterModel() returns a String representing the current connected printer's model.
  @override
  Future<String?> getPrinterModel() async {
    String? printerStatus =
        await methodChannel.invokeMethod<String>('getPrinterModel');
    return printerStatus;
  }

  /// getConnectionType() returns a String representing the protocol that was used to connect to the printer.
  @override
  Future<String?> getConnectionType() async {
    String? connectionType =
        await methodChannel.invokeMethod<String>('getConnectionType');
    return connectionType;
  }

  /// getBatteryLevelPercentage() returns a String representing the battery level percentage of the connected printer.
  @override
  Future<int?> getBatteryLevelPercentage() async {
    int? batteryLevelPercentage =
        await methodChannel.invokeMethod<int>('getBatteryLevelPercentage');
    return batteryLevelPercentage;
  }

  /// getIsAcConnected() returns a Boolean representing if the connect printer is charging.
  @override
  Future<bool?> getIsAcConnected() async {
    bool? isAcConnected =
        await methodChannel.invokeMethod<bool>('getIsAcConnected');
    return isAcConnected;
  }

  /// getSupplyName() returns a String representing the supply installed in the connected printer.
  @override
  Future<String?> getSupplyName() async {
    String? supplyName =
        await methodChannel.invokeMethod<String>('getSupplyName');
    return supplyName;
  }

  /// getSupplyRemainingPercentage() returns a String representing the percentage of supply remaining in the connected printer
  @override
  Future<int?> getSupplyRemainingPercentage() async {
    int? supplyRemainingPercentage =
        await methodChannel.invokeMethod<int>('getSupplyRemainingPercentage');
    return supplyRemainingPercentage;
  }

  /// getSupplyWidth() returns a String representing the width of the supply installed in the connected printer.
  @override
  Future<double?> getSupplyWidth() async {
    double? supplyWidth =
        await methodChannel.invokeMethod<double>('getSupplyWidth');
    return supplyWidth;
  }

  /// getSupplyHeight() returns a String representing the height of the supply installed in the connected printer.
  @override
  Future<double?> getSupplyHeight() async {
    double? supplyHeight =
        await methodChannel.invokeMethod<double>('getSupplyHeight');
    return supplyHeight;
  }

  /// getIsSupplyPreSized() returns a Boolean representing if the supply installed in the connected printer is pre-sized. If false, the supply is continuous.
  @override
  Future<bool?> getIsSupplyPreSized() async {
    bool? isSupplyPreSized =
        await methodChannel.invokeMethod<bool>('getIsSupplyPreSized');
    return isSupplyPreSized;
  }

  /// checkForPartMismatch() returns a Boolean representing if the selected template was not designed with the supply that is installed in the connected printer.
  @override
  Future<bool?> checkForPartMismatch() async {
    bool? partMismatch =
        await methodChannel.invokeMethod<bool>('checkForPartMismatch');
    return partMismatch;
  }

  /// getLastConnectedPrinterName() returns a String representing the name of the last printer that the app connected to.
  ///
  /// This is available to allow automatic connection. Since most users own and use only one printer, this can be called immediately after startPrinterDiscovery().
  /// Therefore, when a printer is discovered with the same name as the "last connected printer" you may choose to call the connectToPrinter(String) method to connect to the familiar printer immediately at start up.
  @override
  Future<String?> getLastConnectedPrinterName() async {
    String? lastConnectedPrinterName =
        await methodChannel.invokeMethod<String>('getLastConnectedPrinterName');
    return lastConnectedPrinterName;
  }

  /// setTemplate(String) takes a file path (i.e. 'assets/MyTemplate.BWT' or 'assets/MyImage.PNG') as a string representing a template file or image.
  ///
  /// This method sets the current template or image being acted upon when setPlaceholderValue(String, String) or print() is called.
  /// Therefore, calling setTemplate(String) again will override any previous template that has been set.
  @override
  Future<void> setTemplate(String filePath) async {
    String? fileExtension = filePath.split(".").lastOrNull;
    if (fileExtension != null) {
      final fileData = await rootBundle.load(filePath);
      final bytes = fileData.buffer.asUint8List();
      final byteData = base64.encode(bytes);
      var sendMap = <String, dynamic>{
        "byteData": byteData,
        "fileExtension": fileExtension
      };
      methodChannel.invokeMethod('setTemplate', sendMap);
    } else {
      debugPrint("INVALID FILE NAME. PLEASE INCLUDE FILE EXTENSION.");
    }
  }

  ///getHaveOwnership() returns a Boolean representing the mobile device's ownership of a nearby M211.
  ///
  /// This will only ever return True when communicating with an M211. It is possible
  /// to fail a connection to an M211, but successfully gain ownership, hence this method's existence.
  @override
  Future<bool?> getHaveOwnership() async {
    bool? haveOwnership =
        await methodChannel.invokeMethod<bool>('getHaveOwnership');
    return haveOwnership;
  }

  /// getTemplateDataNames returns all object names on the current template.
  ///
  /// This can be used to see placeholder names instead of opening Brady Workstation
  /// to see what they are named. These object names can be directly passed into the
  /// setPlaceholderValue to set the value of a placeholder.
  @override
  Future<List<String>> getTemplateDataNames() async {
    String? templateDataNames =
        await methodChannel.invokeMethod<String>('getTemplateDataNames');
    List<String> templateDataNamesList = templateDataNames!.split(', ');
    templateDataNamesList.removeLast();
    return templateDataNamesList;
  }

  /// getTemplateSupplyName() returns the name of the supply that used to design the selected template
  @override
  Future<String?> getTemplateSupplyName() async {
    String? templateSupplyName =
        await methodChannel.invokeMethod<String>('getTemplateSupplyName');
    return templateSupplyName;
  }

  /// setPlaceholderValue(String, String) takes two Strings to set a placeholder with a specified value.
  ///
  /// The first argument must be a String which matches the placeholder's name in Brady Workstation where the template was created.
  /// The second argument must be a String representing the value you want to set to the placeholder.
  @override
  void setPlaceholderValue(String placeholder, String value) {
    var sendMap = <String, dynamic>{"placeholder": placeholder, "value": value};
    methodChannel.invokeMethod('setPlaceholderValue', sendMap);
  }

  ///getPreview(Int) takes an Integer that represents the width in screen pixels that the print preview should display as (on the UI).
  @override
  Future<String?> getPreview(int elementSize) async {
    var sendMap = <String, dynamic>{"elementSize": elementSize};
    String? preview =
        await methodChannel.invokeMethod<String>('getPreview', sendMap);
    return preview;
  }

  /// print() asynchronously sends a print job of the currently set template to the printer that the app is currently connected to and returns a Boolean value afterwards with the result of the print operation (i.e. success or failure).
  @override
  Future<bool?> print(int copies, bool cutAfterEachLabel, bool isCollated,
      bool printTrailer) async {
    var sendMap = <String, dynamic>{
      "copies": copies,
      "cutAfterEachLabel": cutAfterEachLabel,
      "isCollated": isCollated,
      "printTrailer": printTrailer
    };
    bool? printed = await methodChannel.invokeMethod<bool>('print', sendMap);
    return printed;
  }

  /// printBitmapWithDimensions() asynchronously sends a print job of the currently set template or bitmap image to the printer that the app is currently connected to and returns a Boolean value afterwards with the result of the print operation (i.e. success or failure).
  @override
  Future<bool?> printBitmapWithDimensions(
      double bitmapWidthInInches,
      double labelLengthInInches,
      int copies,
      bool cutAfterEachLabel,
      bool isCollated,
      bool printTrailer) async {
    var sendMap = <String, dynamic>{
      "bitmapWidthInInches": bitmapWidthInInches,
      "labelLengthInInches": labelLengthInInches,
      "copies": copies,
      "cutAfterEachLabel": cutAfterEachLabel,
      "isCollated": isCollated,
      "printTrailer": printTrailer
    };
    bool? printed = await methodChannel.invokeMethod<bool>('print', sendMap);
    return printed;
  }

  /// feed() asynchronously makes the connected printer print one empty label and returns a Boolean value afterwards with the result of the feed operation(i.e. success or failure).
  @override
  Future<bool?> feed() async {
    bool? fed = await methodChannel.invokeMethod<bool>('feed');
    return fed;
  }

  /// cut() asynchronously makes the connected printer perform a cut operation (this can be called right after feed() for testing purposes) and returns a Boolean value afterwards with the result of the cut operation (i.e. success or failure).
  @override
  Future<bool?> cut() async {
    bool? cut = await methodChannel.invokeMethod<bool>('cut');
    return cut;
  }

  /// disconnect() asynchronously disconnects from the currently connected printer and returns a Boolean value afterwards with the result of the disconnect operation (i.e. success or failure).
  ///
  /// This will also clear the value of "LastConnectedPrinterName" internally.
  @override
  Future<bool?> disconnect() async {
    bool? disconnected = await methodChannel.invokeMethod<bool>('disconnect');
    return disconnected;
  }

  /// disconnectWithoutForget() asynchronously disconnects from the currently connected printer and returns a Boolean value afterwards with the result of the disconnect operation (i.e. success or failure).
  ///
  /// This will NOT clear the value of "LastConnectedPrinterName" internally.
  @override
  Future<bool?> disconnectWithoutForget() async {
    bool? disconnected =
        await methodChannel.invokeMethod<bool>('disconnectWithoutForget');
    return disconnected;
  }

  /// forgetLastConnectedPrinter() clears the value of "LastConnectedPrinterName internally preventing auto-connections.
  @override
  void forgetLastConnectedPrinter() {
    methodChannel.invokeMethod('forgetLastConnectedPrinter');
  }

  /// getAvailablePrinterUpdates() returns a list of string representing all printer updates that occurred since the last time this method was called.
  @override
  Future<List<String?>> getAvailablePrinterUpdates() async {
    String? printerUpdates =
        await methodChannel.invokeMethod<String>('getAvailablePrinterUpdates');
    List<String> updatesList = printerUpdates!.split(', ');
    updatesList.removeLast();
    return updatesList;
  }
}
